package com.smppw.modaq.infrastructure.util;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.github.junrar.Archive;
import com.github.junrar.exception.RarException;
import com.github.junrar.rarfile.FileHeader;
import org.apache.commons.compress.archivers.ArchiveEntry;
import org.apache.commons.compress.archivers.ArchiveException;
import org.apache.commons.compress.archivers.ArchiveInputStream;
import org.apache.commons.compress.archivers.ArchiveStreamFactory;
import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;
import org.apache.commons.compress.archivers.zip.ZipFile;
import org.apache.commons.io.IOUtils;

import java.io.*;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardCopyOption;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

public class ExcelUtil {
    public static boolean isExcel(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("xls") || fileName.endsWith("xlsx") || fileName.endsWith("XLS") || fileName.endsWith("XLSX"));
    }

    public static boolean isPdf(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("pdf") || fileName.endsWith("PDF"));
    }

    public static boolean isZip(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("zip") || fileName.endsWith("ZIP"));
    }

    public static boolean isHTML(String fileName) {
        return StrUtil.isNotBlank(fileName) && fileName.endsWith("html");
    }

    public static boolean isRAR(String fileName) {
        return StrUtil.isNotBlank(fileName) && (fileName.endsWith("rar") || fileName.endsWith("RAR"));
    }

    public static List<String> extractCompressedFiles(String zipFilePath, String destFilePath) throws IOException, ArchiveException {
        List<String> filePathList = CollUtil.newArrayList();

        File destFile = FileUtil.file(destFilePath).getCanonicalFile();
        if (!destFile.exists()) {
            Files.createDirectories(destFile.toPath());
        }

        try (BufferedInputStream fis = new BufferedInputStream(new FileInputStream(zipFilePath));
             ArchiveInputStream<? extends ArchiveEntry> ais = new ArchiveStreamFactory("UTF-8")
                     .createArchiveInputStream(ArchiveStreamFactory.ZIP, fis)) {
            ArchiveEntry entry;
            int i = 1;
            while ((entry = ais.getNextEntry()) != null) {
                // 跳过不可读条目（如加密文件）
                if (!ais.canReadEntryData(entry)) {
                    continue;
                }
                String name = entry.getName();
                File entryFile = FileUtil.file(destFilePath, name);

                // 安全检查
                if (!entryFile.toPath().normalize().startsWith(destFile.toPath())) {
                    throw new IOException("非法路径: " + name);
                }

                if (entry.isDirectory()) {
                    Files.createDirectories(entryFile.toPath());
                } else {
                    Files.createDirectories(entryFile.getParentFile().toPath());
                    try (OutputStream fos = Files.newOutputStream(entryFile.toPath())) {
                        IOUtils.copy(ais, fos);
                    }
                    filePathList.add(entryFile.getPath());
                }
            }
        } catch (Exception e) {
            if (e.getMessage() != null
                    && (e.getMessage().contains("split")
                    || e.getMessage().contains("volume"))) {
                filePathList.addAll(extractSplitZip(zipFilePath, destFilePath));
            } else {
                throw e;
            }
        }

        return filePathList;
    }

    public static List<String> extractRar(String inputFilePath, String outputDirPath) throws IOException {
        List<String> fileList = new ArrayList<>();
        Path outputPath = Paths.get(outputDirPath).toRealPath();

        try (FileInputStream fis = new FileInputStream(inputFilePath);
             Archive archive = new Archive(fis)) {  // 假设Archive支持try-with-resources

            for (FileHeader entry : archive.getFileHeaders()) {
                // 跳过目录条目（由文件创建逻辑自动生成目录）
                if (entry.isDirectory()) continue;

                // 处理文件名编码和路径
                String fileName = new String(entry.getFileNameByteArray(), StandardCharsets.UTF_8);
                Path targetPath = outputPath.resolve(fileName).normalize();

                // 路径安全检查
                if (!targetPath.startsWith(outputPath)) {
                    throw new IOException("检测到非法路径: " + fileName);
                }

                // 创建父目录
                Files.createDirectories(targetPath.getParent());

                // 处理文件重名
                targetPath = resolveDuplicate(targetPath);

                // 解压文件内容
                try (InputStream entryStream = archive.getInputStream(entry)) {
                    Files.copy(entryStream, targetPath, StandardCopyOption.REPLACE_EXISTING);
                    fileList.add(targetPath.toAbsolutePath().toString());
                }
            }
        } catch (UnsupportedEncodingException | RarException e) {
            throw new IOException("解析RAR文件失败", e);
        }
        return fileList;
    }

    private static Path resolveDuplicate(Path path) {
        if (!Files.exists(path)) return path;
        String base = path.getFileName().toString();
        String name = base.replaceFirst("[.][^.]+$", "");
        String ext = base.substring(name.length());
        int counter = 1;
        while (true) {
            Path newPath = path.resolveSibling(name + "_" + counter + ext);
            if (!Files.exists(newPath)) return newPath;
            counter++;
        }
    }

    public static List<String> extractSplitZip(String zipFilePath, String destFilePath) throws IOException {
        List<String> resultList = ListUtil.list(false);
        File file = new File(zipFilePath);
        try (ZipFile zipFile = ZipFile.builder().setFile(file).get()) {
            Enumeration<ZipArchiveEntry> entries = zipFile.getEntries();
            while (entries.hasMoreElements()) {
                ZipArchiveEntry entry = entries.nextElement();
                // 解压到目标目录
                try (InputStream is = zipFile.getInputStream(entry)) {
                    Path path = Paths.get(destFilePath, entry.getName());
                    FileUtil.del(path);
                    Files.copy(is, path);
                    resultList.add(path.toAbsolutePath().toString());
                }
            }
        }
        return resultList;
    }

    public static void main(String[] args) throws IOException, ArchiveException {
        String zipFilePath = "1.zip";
        String destFilePath = "xx";
        List<String> fileList = extractCompressedFiles(zipFilePath, destFilePath);
        for (String s : fileList) {
            System.out.println(s);
        }
    }
}
